<?php
/*
   +-------------------------------------------------------------------------+
   | Copyright (C) 2004-2013 The Cacti Group                                 |
   |                                                                         |
   | This program is free software; you can redistribute it and/or           |
   | modify it under the terms of the GNU General Public License             |
   | as published by the Free Software Foundation; either version 2          |
   | of the License, or (at your option) any later version.                  |
   |                                                                         |
   | This program is snmpagent in the hope that it will be useful,           |
   | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
   | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
   | GNU General Public License for more details.                            |
   +-------------------------------------------------------------------------+
   | Cacti: The Complete RRDTool-based Graphing Solution                     |
   +-------------------------------------------------------------------------+
   | This code is designed, written, and maintained by the Cacti Group. See  |
   | about.php and/or the AUTHORS file for specific developer information.   |
   +-------------------------------------------------------------------------+
   | http://www.cacti.net/                                                   |
   +-------------------------------------------------------------------------+
*/

/* This mib parser bases on the PHP mib_compiler script of David Eder for
   phpsnmp which bases itself on libsnmp. Both scripts are licensed under the
   GNU Lesser General Public License.

   /**
   * phpsnmp - a PHP SNMP library
   * Copyright (C) 2004 David Eder <david@eder,us>
   *
   * Based on snmp - a Python SNMP library
   * Copyright (C) 2003 Unicity Pty Ltd <libsnmp@unicity.com.au>
   *
   * This library is free software; you can redistribute it and/or
   * modify it under the terms of the GNU Lesser General Public
   * License as published by the Free Software Foundation; either
   * version 2.1 of the License, or (at your option) any later version.
   *
   * This library is distributed in the hope that it will be useful,
   * but WITHOUT ANY WARRANTY; without even the implied warranty of
   * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   * Lesser General Public License for more details.
   *
   * You should have received a copy of the GNU Lesser General Public
   * License along with this library; if not, write to the Free Software
   * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   *
*/
	class snmpagent_catalogue{

		var $parsed = array();
		var $oids = array();

		/**
		 * Constructor
		 */
		function __construct(){
			set_time_limit(0);
			ini_set('memory_limit', '256M');
			error_reporting(E_ALL);
		}

		function add_mib($filename){
			$this->parse_mib(file_get_contents($filename), $filename, true);
		}

		/**
		 * Get Tokens
		 *
		 * @param string $text
		 * @return array
		 */
		function get_tokens($text)
		{
			$in_quote = false;
			$in_comment = false;
			$token = '';
			$tokens = array();
			$length = strlen($text);
			for($i = 0; $i < $length; $i++)
			{
				if($in_quote)
				{
					if($text{$i} == '"')
					{
						$in_quote = false;
						if($token != '')
						{
							$tokens[] = $token;
							$token = '';
						}
					}
					else
					$token .= $text{$i};
				}
				elseif($in_comment)
				{
					if($text{$i} == "\n" || $text{$i} == "\r")
					  $in_comment = false;
				}
				else
				{
					switch($text{$i})
					{
						case ':':
							if($text{$i+1} == ':' && $text{$i+2} == '=')
							{
								if($token != '')
								{
									$tokens[] = $token;
									$token = '';
								}
								$tokens[] = '::=';
								$i += 2;
							}
							else
							$token .= $text{$i};
							break;
						case '.':
							if($text{$i+1} == '.')
							{
								if($token != '')
								{
									$tokens[] = $token;
									$token = '';
								}
								$tokens[] = '..';
								$i++;
							}
							else
							$token .= $text{$i};
							break;
						case ',':
						case ';':
						case '{':
						case '}':
						case '(':
						case ')':
						case '|':
							if($token != '')
							{
								$tokens[] = $token;
								$token = '';
							}
							$tokens[] = $text{$i};
							break;
						case ' ':
						case "\t":
						case "\n":
						case "\r":
							if($token != '')
							{
								$tokens[] = $token;
								$token = '';
							}
							break;
						case '-':
							if($text{$i+1} == '-')
							  $in_comment = true;
							else
							$token .= $text{$i};
							break;
						case '"';
							$in_quote = true;
							break;
						default:
							$token .= $text{$i};
					}
				}
			}
			if($token != '')
				$tokens[] = $token;
			return $tokens;
		}

		/**
		 * Parse simple token
		 *
		 * @param array $tokens
		 * @param integer $index
		 * @param array $allowed
		 * @return array
		 */
		function parse_simple_token($tokens, &$index, $allowed=NULL)
		{
			if(is_array($allowed))
			{
				if(in_array(strtolower($tokens[$index+1]), $allowed))
					return $tokens[++$index];
			}
			elseif(is_null($allowed))
			{
				if($tokens[$index+1] == '{')
					return $this->parse_bracket_token($tokens, ++$index, '{', '}');
				else
					return $tokens[++$index];
			}
			trigger_error("unknown token {$tokens[$index]} {$tokens[$index+1]}", E_USER_ERROR);
			return $tokens[++$index];
		}

		/**
		 * Parse SYNTAX token
		 *
		 * @param array $tokens
		 * @param integer $index
		 * @return array
		 */
		function parse_SYNTAX_token($tokens, &$index)
		{
			$ret = NULL;
			switch($tokens[$index+1])
			{
				case 'SEQUENCE':
					if($tokens[$index+2] == 'OF')
					{
						$index += 2;
						if($tokens[$index+1] == '{')
							$ret = array('SEQUENCE OF'=>$this->parse_bracket_token($tokens, ++$index, '{', '}'));
						else
							$ret = array('SEQUENCE OF'=>$tokens[++$index]);
					}
					break;
				case 'OCTET':
					if($tokens[$index+2] == 'STRING')
					{
						$index += 2;
						if($tokens[$index+1] == '{')
							$ret = array('OCTET STRING'=>$this->parse_bracket_token($tokens, ++$index, '{', '}'));
						elseif($tokens[$index+1] == '(')
							$ret = array('OCTET STRING'=>$this->parse_bracket_token($tokens, ++$index, '(', ')'));
						else
							$ret = 'OCTET STRING';
					}
					break;
				case 'OBJECT':
					if($tokens[$index+2] == 'IDENTIFIER')
						$ret = $tokens[++$index] . ' ' . $tokens[++$index];
					else
						trigger_error("unknown token {$tokens[$index+1]} {$tokens[$index+2]}", E_USER_ERROR);
					break;
				case 'INTEGER':
				case 'Counter':
				case 'Counter32':
				case 'Counter64':
				case 'Integer32':
				case 'Gauge':
				case 'Gauge32':
				case 'TimeStamp':
				case 'TimeTicks':
				case 'PhysAddress':
				case 'IpAddress':
				case 'DateAndTime':
				case 'TimeInterval':
				case 'Unsigned32':
				case 'DisplayString':
					$ret = $tokens[++$index];
					if($tokens[$index+1] == '{')
						$ret = array($ret=>$this->parse_bracket_token($tokens, ++$index, '{', '}'));
					elseif($tokens[$index+1] == '(')
						$ret = array($ret=>$this->parse_bracket_token($tokens, ++$index, '(', ')'));
					break;
				default:
					$ret = $tokens[++$index];
					if($tokens[$index+1] == '{')
						$ret = array($ret=>$this->parse_bracket_token($tokens, ++$index, '{', '}'));
					elseif($tokens[$index+1] == '(')
						$ret = array($ret=>$this->parse_bracket_token($tokens, ++$index, '(', ')'));
					break;
			}
			return $ret;
		}

		/**
		 * Parse bracket token
		 *
		 * @param array $tokens
		 * @param integer $index
		 * @param integer $start
		 * @param integer $end
		 * @return array
		 */
		function parse_bracket_token($tokens, &$index, $start, $end)
		{
			$begin = $index + 1;
			while($index + 1 < count($tokens) && $tokens[$index] != $end)
			{
				$index++;
				if($tokens[$index] == $start)
				{
					$this->parse_bracket_token($tokens, $index, $start, $end);
					$index++;
				}
			}
			return array_slice($tokens, $begin, $index - $begin);
		}

		/**
		 * Parse a MIB file
		 *
		 * @param string $mibtext
		 * @param boolean $full
		 */
		function parse_mib($mibtext, $mib_name, $full=false)
		{
			$tokens = $this->get_tokens($mibtext);
			$cnt = count($tokens);
			$rec = array();
			for($index = 0; $index < $cnt; $index++)
			{
				if($tokens[$index] == 'OBJECT-IDENTITY' || $tokens[$index] == 'OBJECT-TYPE' || $tokens[$index] == 'MODULE-IDENTITY')
				{
					if($tokens[$index-1] != ',' && $tokens[$index+1] != 'FROM' && $tokens[$index+1] != 'MACRO')
					{
						if(isset($rec['NAME']) && isset($rec['VALUE'])) $this->parsed[] = $rec;
						$rec = array('NAME'=>$tokens[$index-1], 'MIB'=>$mib_name );
					}
				}
				elseif($tokens[$index] == 'OBJECT')
				{
					if($tokens[$index+1] == 'IDENTIFIER' && $tokens[$index-1] != '(' && $tokens[$index-1] != '::=' && $tokens[$index-1] != 'SYNTAX' && $tokens[$index-2] != '(')
					{
						if(isset($rec['NAME']) && isset($rec['VALUE'])) $this->parsed[] = $rec;
						$rec = array('NAME'=>$tokens[$index-1], 'MIB'=>$mib_name);
					}
				}
				elseif($tokens[$index] == '{')
					$this->parse_bracket_token($tokens, $index, '{', '}');
				elseif(isset($rec['NAME']))
				{
					if($tokens[$index] == '::=')
					{
						$rec['VALUE'] = $this->parse_simple_token($tokens, $index);
						$this->parsed[] = $rec;
						$rec = array();
					}
					elseif($full)
					{
						if($tokens[$index] == 'ACCESS')
							$rec['ACCESS'] = $this->parse_simple_token($tokens, $index, array('read-only', 'not-accessible', 'read-write'));
						elseif($tokens[$index] == 'DEFVAL')
							$rec['DEFVAL'] = $this->parse_simple_token($tokens, $index);
						elseif($tokens[$index] == 'DESCRIPTION')
						{
							//$rec['DESCRIPTION'] = $tokens[++$index];
							$rec['DESCRIPTION'] = $this->parse_simple_token($tokens, $index);
						}elseif($tokens[$index] == 'INDEX')
							$rec['INDEX'] = $this->parse_simple_token($tokens, $index);
						elseif($tokens[$index] == 'MAX-ACCESS')
							$rec['MAX-ACCESS'] = $this->parse_simple_token($tokens, $index, array('read-only', 'not-accessible', 'read-write', 'read-create', 'accessible-for-notify'));
						elseif($tokens[$index] == 'REFERENCE')
							$rec['REFERENCE'] = $this->parse_simple_token($tokens, $index);
						elseif($tokens[$index] == 'STATUS')
							$rec['STATUS'] = $this->parse_simple_token($tokens, $index, array('current', 'deprecated', 'obsolete', 'mandatory'));
						elseif($tokens[$index] == 'SYNTAX')
							$rec['SYNTAX'] = $this->parse_SYNTAX_token($tokens, $index);
						elseif($tokens[$index] == 'UNITS')
							$rec['UNITS'] = $this->parse_simple_token($tokens, $index);
					}
				}
			}
			if(isset($rec['NAME']) && isset($rec['VALUE'])) $this->parsed[] = $rec;


		}

		function generate(){
			$this->oids['enterprises'] = array('oid' => '.1.3.6.1.4.1');
			foreach($this->parsed as $object) {
				if(isset($object['VALUE'][0]) && !is_numeric($object['VALUE'][0])) {
					if(isset($object['VALUE'][1]) && is_numeric($object['VALUE'][1])) {
						$oid = $this->oids[$object['VALUE'][0]]['oid'] . "." . $object['VALUE'][1];
						$syntax = NULL;
						if(isset($object['SYNTAX'])) {
							$syntax = is_array($object['SYNTAX']) ? key($object['SYNTAX']) : $object['SYNTAX'];
						}
						$otype = strtoupper(substr($object['NAME'], -5));
						$parent_otype = strtoupper(substr($object['VALUE'][0], -5));
						if(in_array($otype, array("TABLE", "ENTRY")) | in_array($parent_otype, array("TABLE", "ENTRY"))) {
							$otype = "";
						}else {
							if($syntax == NULL) {
								$otype = "";
							}else {
								$otype = "DATA";
								$oid .= ".0";
							}
						}
						$this->oids[$object['NAME']] = array(
							'oid' => $oid,
							'max-access' => (isset($object['MAX-ACCESS'])? $object['MAX-ACCESS'] : 'not-accessible'),
							'syntax' => $syntax,
							'otype' => $otype
						);
					}
				}
			}
			unset($this->oids['enterprises']);
			//return $oids;
		}

	}



  //$mc = new mib_compiler();
  //$mc->add_mibs('./mibs');
  //$mc->add_mibs('/proj/netops/cacti/test/phpsnmp/mibs');
  //$mc->compile();

?>
