#!/usr/bin/php
<?php
/*
   +-------------------------------------------------------------------------+
   | Copyright (C) 2004-2013 The Cacti Group                                 |
   |                                                                         |
   | This program is free software; you can redistribute it and/or           |
   | modify it under the terms of the GNU General Public License             |
   | as published by the Free Software Foundation; either version 2          |
   | of the License, or (at your option) any later version.                  |
   |                                                                         |
   | This program is snmpagent in the hope that it will be useful,           |
   | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
   | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
   | GNU General Public License for more details.                            |
   +-------------------------------------------------------------------------+
   | Cacti: The Complete RRDTool-based Graphing Solution                     |
   +-------------------------------------------------------------------------+
   | This code is designed, written, and maintained by the Cacti Group. See  |
   | about.php and/or the AUTHORS file for specific developer information.   |
   +-------------------------------------------------------------------------+
   | http://www.cacti.net/                                                   |
   +-------------------------------------------------------------------------+
*/

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}

/* we are not talking to the browser */
$no_http_headers = true;

/* let's report all errors */
error_reporting(E_ALL);

/* allow the script to hang around waiting for connections. */
set_time_limit(0);

chdir(dirname(__FILE__));
include_once("../../include/global.php");

/* translate well-known textual conventions and SNMP base types to net-snmp */
$smi_base_datatypes = array(
	"integer" 			=> "INTEGER",
	"integer32"			=> "Integer32",
	"unsigned32" 		=> "Unsigned32",
	"gauge" 			=> "Gauge",
	"gauge32" 			=> "Gauge32",
	"counter" 			=> "Counter",
	"counter32" 		=> "Counter32",
	"counter64" 		=> "Counter64",
	"timeticks" 		=> "TimeTicks",
	"octect string" 	=> "OCTET STRING",
	"opaque"			=> "Opaque",
	"object identifier" => "OBJECT IDENTIFIER",
	"ipaddress" 		=> "IpAddress",
	"networkaddress" 	=> "IpAddress",
	"bits" 				=> "OCTET STRING",
	"displaystring" 	=> "STRING",
	"physaddress" 		=> "OCTET STRING",
	"macaddress" 		=> "OCTET STRING",
	"truthvalue" 		=> "INTEGER",
	"testandincr" 		=> "Integer32",
	"autonomoustype" 	=> "OBJECT IDENTIFIER",
	"variablepointer" 	=> "OBJECT IDENTIFIER",
	"rowpointer" 		=> "OBJECT IDENTIFIER",
	"rowstatus" 		=> "INTEGER",
	"timestamp" 		=> "TimeTicks",
	"timeinterval" 		=> "Integer32",
	"dateandtime" 		=> "STRING",
	"storagetype" 		=> "INTEGER",
	"tdomain" 			=> "OBJECT IDENTIFIER",
	"taddress" 			=> "OCTET STRING"
);

$data	= false;
$eol	= "\n";

while(1) {
	$input = trim(fgets(STDIN));

	switch($input) {
		case "":
			exit;
			break;
		case "PING":
			print "PONG"  . $eol;
			break;
		case "get":
			$oid = trim(fgets(STDIN));
			$data = db_fetch_row("SELECT oid, LOWER(type) as type, value FROM plugin_snmpagent_cache WHERE `max-access` IN ('read-only', 'read-write') AND oid='" . $oid . "'");

			if($data) {
				$data["type"] = isset($smi_base_datatypes[$data["type"]]) ? $smi_base_datatypes[$data["type"]] : "INTEGER";
				print $data["oid"] . $eol;
				print $data["type"] . $eol;
				print $data["value"] . $eol;

			}else {
				print "ERROR" . $eol;
			}
			break;
		case "getnext":
			$oid = trim(fgets(STDIN));
			$data = db_fetch_assoc("SELECT oid FROM plugin_snmpagent_cache WHERE `oid` = '" . $oid . "' OR (`max-access` IN ('read-only', 'read-write') AND `otype` = 'DATA')");
			if($data && sizeof($data)>0) {
				$oids = array();
				foreach($data as $key => $object) {
					$oids[] = $object["oid"];
				}
				natsort($oids);
				$oids = array_values($oids);
				$init_key = array_search($oid, $oids);
				if($init_key !== false) {
					if(isset($oids[$init_key+1])) {
						$next = $oids[$init_key+1];
						$data = db_fetch_row("SELECT oid, LOWER(type) as type, value FROM plugin_snmpagent_cache WHERE oid='" . $next . "'");
						$data["type"] = isset($smi_base_datatypes[$data["type"]]) ? $smi_base_datatypes[$data["type"]] : "INTEGER";
						print $data["oid"] . $eol;
						print $data["type"] . $eol;
						print $data["value"] . $eol;
					}else {
						print "NONE" . $eol;
					}
				}else {
					$oid .= ".0";
					if(in_array($oid, $oids)) {
						$data = db_fetch_row("SELECT oid, LOWER(type) as type, value FROM plugin_snmpagent_cache WHERE oid='" . $oid . "'");
						$data["type"] = isset($smi_base_datatypes[$data["type"]]) ? $smi_base_datatypes[$data["type"]] : "INTEGER";
						print $data["oid"] . $eol;
						print $data["type"] . $eol;
						print $data["value"] . $eol;
					}else {
						print "NONE" . $eol;
					}
				}

			}else {
				print "NONE" . $eol;
			}
			break;
	}
}
?>