<?php
/*
   +-------------------------------------------------------------------------+
   | Copyright (C) 2004-2013 The Cacti Group                                 |
   |                                                                         |
   | This program is free software; you can redistribute it and/or           |
   | modify it under the terms of the GNU General Public License             |
   | as published by the Free Software Foundation; either version 2          |
   | of the License, or (at your option) any later version.                  |
   |                                                                         |
   | This program is snmpagent in the hope that it will be useful,           |
   | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
   | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
   | GNU General Public License for more details.                            |
   +-------------------------------------------------------------------------+
   | Cacti: The Complete RRDTool-based Graphing Solution                     |
   +-------------------------------------------------------------------------+
   | This code is designed, written, and maintained by the Cacti Group. See  |
   | about.php and/or the AUTHORS file for specific developer information.   |
   +-------------------------------------------------------------------------+
   | http://www.cacti.net/                                                   |
   +-------------------------------------------------------------------------+
*/

function plugin_snmpagent_install() {

	/* updates based on user actions */
	api_plugin_register_hook('snmpagent', 'global_settings_update', 	'snmpagent_global_settings_update', 	'setup.php');
	api_plugin_register_hook('snmpagent', 'device_action_bottom', 		'snmpagent_device_action_bottom', 		'setup.php');
	api_plugin_register_hook('snmpagent', 'api_device_new', 			'snmpagent_api_device_new', 			'setup.php');
	api_plugin_register_hook('snmpagent', 'data_source_action_bottom', 	'snmpagent_data_source_action_bottom', 	'setup.php');
	api_plugin_register_hook('snmpagent', 'graphs_action_bottom', 		'snmpagent_graphs_action_bottom', 		'setup.php');
	/* poller related updates */
	api_plugin_register_hook('snmpagent', 'poller_bottom', 				'snmpagent_poller_bottom', 				'setup.php');
	api_plugin_register_hook('snmpagent', 'host_status_update', 		'snmpagent_host_status_update', 		'setup.php');
	api_plugin_register_hook('snmpagent', 'cacti_stats_update', 		'snmpagent_cacti_stats_update', 		'setup.php');

	snmpagent_setup_table_new ();
	snmpagent_cache_rebuild();
}

function plugin_snmpagent_uninstall () {
	/* destroy SNMPAgent's snmpcache */
	db_execute("DROP TABLE IF EXISTS `plugin_snmpagent_cache`;");
	db_execute("DROP TABLE IF EXISTS `plugin_snmpagent_mibs`;");
	return true;
}

function plugin_snmpagent_check_config () {
	/* Here we will check to ensure everything is configured */
	snmpagent_check_upgrade();
	return true;
}

function plugin_snmpagent_upgrade () {
	/* Here we will upgrade to the newest version */
	snmpagent_check_upgrade();
	return false;
}

function plugin_snmpagent_version () {
	return snmpagent_version();
}

function snmpagent_check_upgrade () {
	global $config;

	$files = array('index.php', 'plugins.php');
	if (isset($_SERVER['PHP_SELF']) && !in_array(basename($_SERVER['PHP_SELF']), $files)) {
		return;
	}

	$current = plugin_snmpagent_version();
	$current = $current['version'];
	$old = db_fetch_row("SELECT * FROM plugin_config WHERE directory='snmpagent'");
	if (sizeof($old) && $current != $old["version"]) {
		/* if the plugin is installed and/or active */
		if ($old["status"] == 1 || $old["status"] == 4) {
			/* re-register the hooks */
			plugin_snmpagent_install();
			/* perform a database upgrade */
			snmpagent_database_upgrade();
		}

		/* update the plugin information */
		$info = plugin_snmpagent_version();
		$id = db_fetch_cell("SELECT id FROM plugin_config WHERE directory='snmpagent'");
		db_execute("UPDATE plugin_config
					SET name='" . $info["longname"] . "',
						author='" . $info["author"] . "',
						webpage='" . $info["homepage"] . "',
						version='" . $info["version"] . "'
					WHERE id='$id'");
	}
}

function snmpagent_database_upgrade () {
}

function snmpagent_check_dependencies() {
	global $plugins, $config;
	return true;
}

function snmpagent_setup_table_new () {
	$sql = "CREATE TABLE IF NOT EXISTS `plugin_snmpagent_cache` (
				`oid` varchar(255) NOT NULL,
				`name` varchar(255) NOT NULL,
				`mib` varchar(255) NOT NULL,
				`type` varchar(255) NOT NULL DEFAULT '',
				`otype` varchar(255) NOT NULL DEFAULT '',
				`max-access` varchar(255) NOT NULL DEFAULT 'not-accessible',
				`value` varchar(255) NOT NULL DEFAULT '',
				KEY `oid` (`oid`),
				KEY `name` (`name`),
				KEY `mib` (`mib`)
			) ENGINE=MyISAM  DEFAULT CHARSET=utf8";
	db_execute($sql);
	$sql = "CREATE TABLE IF NOT EXISTS `plugin_snmpagent_mibs` (
				`id` int(8) NOT NULL AUTO_INCREMENT,
				`name` varchar(32) NOT NULL DEFAULT '',
				`hook` varchar(64) NOT NULL DEFAULT '',
				`file` varchar(255) NOT NULL DEFAULT '',
				PRIMARY KEY (`id`),
				KEY `hook` (`hook`)
			) ENGINE=MyISAM  DEFAULT CHARSET=utf8";
	db_execute($sql);
}

function snmpagent_version () {
	return array(
		'name'     => 'snmpagent',
		'version'  => '0.1',
		'longname' => 'Cacti SNMP Agent',
		'author'   => 'The Cacti Group',
		'homepage' => 'http://www.cacti.net',
		'email'    => '',
		'url'      => ''
	);
}

function snmpagent_cacti_stats_update($data){
	/* refresh total stats */
	snmpagent_cachedMibGroup_refresh('CACTI-SMI', array('cactiStatsTotalsDevices', 'cactiStatsTotalsDataSources', 'cactiStatsTotalsGraphs'));

	/* local polling stats  - not sure how this will be handled later in a distributed environment. */
	snmpagent_cache_update('CACTI-SMI', 'cactiStatsLocalPollerRunTime',$data[0]);
	$values = array(
		"cactiStatsPollerRunTime" => $data[0],
		"cactiStatsPollerConcurrentProcesses" => $data[2],
		"cactiStatsPollerThreads" => $data[3],
		"cactiStatsPollerHosts" => $data[4],
		"cactiStatsPollerHostsPerProcess" => $data[5],
		"cactiStatsPollerItems" => $data[6],
		"cactiStatsPollerRrrdsProcessed" => $data[7],
		"cactiStatsPollerUtilization" => round($data[0]/read_config_option("poller_interval", true)*100, 10)
	);
	snmpagent_cacheTableEntry_update('CACTI-SMI', 'cactiStatsPollerTable', 1, $values);
	snmpagent_cachedMibObject_refresh('CACTI-SMI','cactiStatsLastUpdate');
}

function snmpagent_global_settings_update(){
	/* update global settings related objects */
	$objects = array(
			"cactiApplVersion",
			"cactiApplSnmpVersion",
			"cactiApplRrdtoolVersion",
			"cactiApplPollerEnabled",
			"cactiApplPollerType",
			"cactiApplPollerInterval",
			"cactiApplPollerMaxProcesses",
			"cactiApplPollerLoadBalance",
			"cactiApplSpineMaxThreads",
			"cactiApplSpineScriptServers",
			"cactiApplSpineScriptTimeout",
			"cactiApplSpineMaxOids",
			"cactiApplLastUpdate"
	);
	snmpagent_cachedMibGroup_refresh('CACTI-SMI', $objects);
}

function snmpagent_host_status_update($data){

	$index = $data["host_id"];
	$values = array(
		"cactiApplDeviceStatus" => $data["host_data"]["status"],
		"cactiApplDeviceEventCount" => $data["host_data"]["status_event_count"],
		"cactiApplDeviceFailDate" => $data["host_data"]["status_fail_date"],
		"cactiApplDeviceRecoveryDate" => $data["host_data"]["status_rec_date"],
		"cactiApplDeviceLastError" => $data["host_data"]["status_last_error"]
	);
	snmpagent_cacheTableEntry_update('CACTI-SMI', 'cactiApplDeviceTable', $index, $values);

	$values = array(
		"cactiStatsDeviceMinTime" => $data["host_data"]["min_time"],
		"cactiStatsDeviceMaxTime" => $data["host_data"]["max_time"],
		"cactiStatsDeviceCurTime" => $data["host_data"]["cur_time"],
		"cactiStatsDeviceAvgTime" => $data["host_data"]["avg_time"],
		"cactiStatsDeviceTotalPolls" => $data["host_data"]["total_polls"],
		"cactiStatsDeviceFailedPolls" => $data["host_data"]["failed_polls"],
		"cactiStatsDeviceAvailability" => $data["host_data"]["availability"]
	);
	snmpagent_cacheTableEntry_update('CACTI-SMI', 'cactiStatsDeviceTable', $index, $values);
	snmpagent_cachedMibGroup_refresh('CACTI-SMI', array('cactiApplLastUpdate', 'cactiStatsLastUpdate'));
}

function snmpagent_api_device_new($device){
	/* add device to cactiApplDeviceTable and cactiStatsDeviceTable*/
	$device_data = db_fetch_row("SELECT * FROM `host` WHERE id = " . $device["id"]);

	$appl_values = array(
		"cactiApplDeviceIndex" => $device_data["id"],
		"cactiApplDeviceDescription" => $device_data["description"],
		"cactiApplDeviceHostname" => $device_data["hostname"],
		"cactiApplDeviceStatus" => $device_data["status"],
		"cactiApplDeviceEventCount" => $device_data["status_event_count"],
		"cactiApplDeviceFailDate" => $device_data["status_fail_date"],
		"cactiApplDeviceRecoveryDate" => $device_data["status_rec_date"],
		"cactiApplDeviceLastError" => $device_data["status_last_error"],
	);

	$stats_values = array(
		"cactiStatsDeviceIndex" => $device_data["id"],
		"cactiStatsDeviceHostname" => $device_data["hostname"],
		"cactiStatsDeviceMinTime" => $device_data["min_time"],
		"cactiStatsDeviceMaxTime" => $device_data["max_time"],
		"cactiStatsDeviceCurTime" => $device_data["cur_time"],
		"cactiStatsDeviceAvgTime" => $device_data["avg_time"],
		"cactiStatsDeviceTotalPolls" => $device_data["total_polls"],
		"cactiStatsDeviceFailedPolls" => $device_data["failed_polls"],
		"cactiStatsDeviceAvailability" => $device_data["availability"]
	);

	if(!snmpagent_cacheTableEntry_exists("CACTI-SMI","cactiApplDeviceTable", $device["id"])) {
		snmpagent_cacheTableEntry_insert("CACTI-SMI", "cactiApplDeviceTable", $device["id"], $appl_values );
		snmpagent_cacheTableEntry_insert("CACTI-SMI", "cactiStatsDeviceTable", $device["id"], $stats_values );
	}else {
		snmpagent_cacheTableEntry_update("CACTI-SMI", "cactiApplDeviceTable", $device["id"], $appl_values );
		snmpagent_cacheTableEntry_update("CACTI-SMI", "cactiStatsDeviceTable", $device["id"], $stats_values );
	}

	/* update device counter and timestamps */
	snmpagent_cachedMibGroup_refresh('CACTI-SMI', array('cactiStatsTotalsDevices', 'cactiApplLastUpdate', 'cactiStatsLastUpdate'));

}

function snmpagent_data_source_action_bottom($data){
	$action = $data[0];
	if($action == "1") {
		/* delete data sources */
		snmpagent_cachedMibGroup_refresh('CACTI-SMI', array('cactiStatsTotalsDataSources', 'cactiStatsTotalsGraphs', 'cactiStatsLastUpdate'));
	}elseif($action == "4") {
		/* duplicate data sources */
		snmpagent_cachedMibGroup_refresh('CACTI-SMI', array('cactiStatsTotalsDataSources', 'cactiStatsLastUpdate'));
	}
}

function snmpagent_graphs_action_bottom($data){
	$action = $data[0];
	if($action == "1") {
		/* delete graphs */
		snmpagent_cachedMibGroup_refresh('CACTI-SMI', array('cactiStatsTotalsDataSources', 'cactiStatsTotalsGraphs', 'cactiStatsLastUpdate'));
	}elseif($action == "3") {
		/* duplicate graphs */
		snmpagent_cachedMibGroup_refresh('CACTI-SMI', array('cactiStatsTotalsGraphs', 'cactiStatsLastUpdate'));
	}
}

function snmpagent_device_action_bottom($data){
	$action = $data[0];
	$selected_items = $data[1];
	/* selected items are give by the user - do not forget to validate them! */
	if(sizeof($selected_items)>0) {
		switch($action){
			case "1":
				/* delete devices */
				foreach($selected_items as $device_id) {
					if(is_numeric($device_id)) {
						/* remove all device entries */
						snmpagent_cacheTableEntry_delete('CACTI-SMI', 'cactiApplDeviceTable', $device_id);
						snmpagent_cacheTableEntry_delete('CACTI-SMI', 'cactiStatsDeviceTable', $device_id);
					}
				}
				/* update total statistics */
				snmpagent_cachedMibGroup_refresh('CACTI-SMI', array('cactiStatsTotalsDevices', 'cactiStatsTotalsDataSources', 'cactiStatsTotalsGraphs', 'cactiStatsLastUpdate'));
				break;
			case "2":
				/* enable devices */
				foreach($selected_items as $device_id) {
					if(is_numeric($device_id)) {
						$device_status = db_fetch_cell("SELECT status FROM host WHERE id = " . $device_id);
						snmpagent_cacheTableEntry_update('CACTI-SMI','cactiApplDeviceTable', $device_id, array('cactiApplDeviceStatus' => $device_status));
					}
				}
				snmpagent_cachedMibObject_refresh('CACTI-SMI','cactiApplLastUpdate');
				break;
			case "3":
				/* disable devices */
				foreach($selected_items as $device_id) {
					if(is_numeric($device_id)) {
						$device_status = db_fetch_cell("SELECT status FROM host WHERE id = " . $device_id);
						snmpagent_cacheTableEntry_update('CACTI-SMI','cactiApplDeviceTable', $device_id, array('cactiApplDeviceStatus' => 4));
					}
				}
				snmpagent_cachedMibObject_refresh('CACTI-SMI','cactiApplLastUpdate');
				break;
			case "5":
				/* clear device statisitics */
				$values = array(
					'cactiStatsDeviceMinTime' => '9.99999',
					'cactiStatsDeviceMaxTime' => '0',
					'cactiStatsdeviceCurTime' => '0',
					'cactiStatsDeviceAvgTime' => '0',
					'cactiStatsDeviceTotalPolls' => '0',
					'cactiStatsDeviceFailedPolls' => '0',
					'cactiStatsDeviceAvailability' => '100'
				);
				foreach($selected_items as $device_id) {
					if(is_numeric($device_id)) {
						snmpagent_cacheTableEntry_update('CACTI-SMI', 'cactiStatsDeviceTable', $device_id, $values);
					}
				}
				snmpagent_cachedMibObject_refresh('CACTI-SMI','cactiStatsLastUpdate');
				break;

			default:
				/* nothing to do */
				;
		} //switch
	}
}

function snmpagent_totalsdevicestatus_update($update = true){
	$counters = db_fetch_assoc("SELECT COUNT(*) as counter, status FROM `host` WHERE `disabled` != 'on' GROUP BY status");
	if($counters && sizeof($counters)>0) {
		foreach($counters as $counter) {
			//snmpagent_cacheTableEntry_update('CACTI_SMI', 'cactiStatsTotalsDeviceStatusTable', 0, array('cacti'))
		}
	}
	//	snmpagent_cacheTableEntry_update( 'CACTI_SMI', 'cactiStatsTotalsDeviceStatusTable', 0, array('cacti'))
}

function snmpagent_poller_bottom() {
	global $plugins_system;

	/* update total device stats table */
	snmpagent_cachedMibTable_refresh('CACTI-SMI', 'cactiStatsTotalsDeviceStatusTable');

	/* get a list of all plugins available on that system */
	$pluginslist = snmpagent_get_pluginslist();
	/* truncate plugin mib table */
	snmpagent_cacheTable_truncate('CACTI-SMI', 'cactiApplPluginTable');
	/* refill plugin mib table */
	if($pluginslist && sizeof($pluginslist)>0) {
		$i = 1;
		foreach($pluginslist as $plugin) {
			$values = array(
				"cactiApplPluginIndex" => $i,
				"cactiApplPluginType" => (in_array($plugin["directory"], $plugins_system) ? 1 : 2),
				"cactiApplPluginName" => $plugin["directory"],
				"cactiApplPluginStatus" => $plugin["status"],
				"cactiApplPluginVersion" => $plugin["version"]
			);
			snmpagent_cacheTableEntry_insert("CACTI-SMI", "cactiApplPluginTable", $i, $values );
			$i++;
		}
	}

	$recache_stats = db_fetch_cell("SELECT value FROM settings WHERE name = 'stats_recache'");
	if($recache_stats) {
		list($time, $hosts) = explode(" ", $recache_stats);
		$time = str_replace("RecacheTime:", "", $time);
		$hosts = str_replace("HostsRecached:", "", $hosts);
	}

	snmpagent_cache_update('CACTI-SMI', 'cactiStatsRecacheTime', $time);
	snmpagent_cache_update('CACTI-SMI', 'cactiStatsRecachedHosts', $hosts);
	snmpagent_cachedMibObject_refresh('CACTI-SMI', 'cactiStatsLastUpdate');
}

function snmpagent_get_pluginslist(){
	global $config, $plugins, $plugins_system;
	/* update the list of known plugins only once per polling cycle. In all other cases we would
	   have to create too many new hooks to update that MIB table just in time.
	   We have to do the same like function plugins_load_temp_table(), which will not be available
	   during the execution of that function. */

	$pluginslist = array();
	$registered_plugins = db_fetch_assoc('SELECT * FROM plugin_config ORDER BY name');
	foreach ($registered_plugins as $t) {
		$pluginslist[$t['directory']] = $t;
	}

	$path = $config['base_path'] . '/plugins/';
	$dh = opendir($path);
	if ($dh !== false) {
		while (($file = readdir($dh)) !== false) {
			if ((is_dir("$path/$file")) && (file_exists("$path/$file/setup.php")) && (!array_key_exists($file, $pluginslist))) {
				include_once("$path/$file/setup.php");
				if (!function_exists('plugin_' . $file . '_install') && function_exists($file . '_version')) {
					$function = $file . '_version';
					$cinfo = $function();
					if (!isset($cinfo['author']))   $cinfo['author']   = 'Unknown';
					if (!isset($cinfo['homepage'])) $cinfo['homepage'] = 'Not Stated';
					if (isset($cinfo['webpage']))   $cinfo['homepage'] = $cinfo['webpage'];
					if (!isset($cinfo['longname'])) $cinfo['longname'] = ucfirst($file);
					$cinfo['status'] = -2; /* old PIA -- disabled */
					if (in_array($file, $plugins)) {
						$cinfo['status'] = -1; /* old PIA -- enabled */
					}
					$cinfo['directory'] = $file;
					$pluginslist[$file] = $cinfo;

				} elseif (function_exists('plugin_' . $file . '_install') && function_exists('plugin_' . $file . '_version')) {
					$function = $file . '_version';
					$cinfo = $function();
					$cinfo['status'] = 0;
					if (!isset($cinfo['author']))   $cinfo['author']   = 'Unknown';
					if (!isset($cinfo['homepage'])) $cinfo['homepage'] = 'Not Stated';
					if (isset($cinfo['webpage']))   $cinfo['homepage'] = $cinfo['webpage'];
					if (!isset($cinfo['longname'])) $cinfo['homepage'] = ucfirst($file);
					$cinfo['directory'] = $file;
					$pluginslist[$file] = $cinfo;
				}
			}
		}
		closedir($dh);
	}
	return $pluginslist;
}

/**
 * snmpagent_cache_install()
 * Generates a SNMP caching tables reflecting all objects of the Cacti MIB
 * @return
 */
function snmpagent_cache_rebuild(){
	global $config, $plugins_system;
	include_once($config["base_path"] . '/plugins/snmpagent/lib/mib_parser.php');

	db_execute("TRUNCATE `plugin_snmpagent_cache`");

	$mc = new snmpagent_catalogue();
	//$mc->add_mibs('./mibs');
	$mc->add_mib($config["base_path"] . '/plugins/snmpagent/mibs/CACTI-SMI.txt');
	$mc->generate();
	if(isset($mc->oids)) {
		print "<pre>";
		foreach($mc->oids as $object_name => $object_params) {
			db_execute("INSERT INTO `plugin_snmpagent_cache` (`oid`, `name`, `mib`, `type`, `otype`, `max-access`) VALUES ('"
						. $object_params["oid"] . "','"
						. $object_name . "','"
						. 'CACTI-SMI' . "','"
						. $object_params["syntax"] . "','"
						. $object_params["otype"] . "','"
						. $object_params["max-access"] . "')");
		}
	}

	/* now fill up the cache with global data */
	snmpagent_global_settings_update();

	/* update total device stats table */
	snmpagent_cachedMibTable_refresh('CACTI-SMI', 'cactiStatsTotalsDeviceStatusTable');

	/* add pollers of a distributed system (future) */
	$pollers = db_fetch_assoc("SELECT id FROM poller ORDER BY id ASC");
	if($pollers && sizeof($pollers)>0) {
		foreach($pollers as $poller){
			$poller_data = db_fetch_row("SELECT * FROM poller WHERE id = " . $poller["id"]);
		}
	}else {
		/* this is NOT a distributed system, but is should have at least one local poller. */
		$poller_lastrun = read_config_option("poller_lastrun", true);
		$values = array(
			"cactiApplPollerIndex" => 1,
			"cactiApplPollerHostname" => "localhost",
			"cactiApplPollerIpAddress" => "127.0.0.1",
			"cactiApplPollerLastUpdate" => $poller_lastrun
		);
		snmpagent_cacheTableEntry_insert( "CACTI-SMI", "cactiApplPollerTable", 1, $values );
		$values = array( "cactiStatsPollerIndex" => 1, "cactiStatsPollerHostname" => "localhost", "cactiStatsPollerMethod" => read_config_option("poller_type", true));
		snmpagent_cacheTableEntry_insert( "CACTI-SMI", "cactiStatsPollerTable", 1, $values );
	}

	/* add devices as device entries to the snmp cache */
	$devices = db_fetch_assoc("SELECT id FROM host ORDER BY id ASC");
	if($devices && sizeof($devices)>0) {
		foreach($devices as $device) {
			$device_data = db_fetch_row("SELECT * FROM host WHERE id = " . $device["id"]);
			/* add device to cactiApplDeviceTable */
			$values = array(
				"cactiApplDeviceIndex" => $device_data["id"],
				"cactiApplDeviceDescription" => $device_data["description"],
				"cactiApplDeviceHostname" => $device_data["hostname"],
				"cactiApplDeviceStatus" => ($device_data["disabled"] == 'on') ? 4 : $device_data["status"],
				"cactiApplDeviceEventCount" => $device_data["status_event_count"],
				"cactiApplDeviceFailDate" => $device_data["status_fail_date"],
				"cactiApplDeviceRecoveryDate" => $device_data["status_rec_date"],
				"cactiApplDeviceLastError" => $device_data["status_last_error"],
			);
			snmpagent_cacheTableEntry_insert("CACTI-SMI", "cactiApplDeviceTable", $device["id"], $values );

			/* add device to cactiStatsDeviceTable */
			$values = array(
				"cactiStatsDeviceIndex" => $device_data["id"],
				"cactiStatsDeviceHostname" => $device_data["hostname"],
				"cactiStatsDeviceMinTime" => $device_data["min_time"],
				"cactiStatsDeviceMaxTime" => $device_data["max_time"],
				"cactiStatsDeviceCurTime" => $device_data["cur_time"],
				"cactiStatsDeviceAvgTime" => $device_data["avg_time"],
				"cactiStatsDeviceTotalPolls" => $device_data["total_polls"],
				"cactiStatsDeviceFailedPolls" => $device_data["failed_polls"],
				"cactiStatsDeviceAvailability" => $device_data["availability"]
			);
			snmpagent_cacheTableEntry_insert("CACTI-SMI", "cactiStatsDeviceTable", $device["id"], $values );
		}
	}

	/* get a list of all plugins available on that system */
	$pluginslist = snmpagent_get_pluginslist();
	if($pluginslist && sizeof($pluginslist)>0) {
		$i = 1;
		foreach($pluginslist as $plugin) {
			$values = array(
				"cactiApplPluginIndex" => $i,
				"cactiApplPluginType" => (in_array($plugin["directory"], $plugins_system) ? 1 : 2),
				"cactiApplPluginName" => $plugin["directory"],
				"cactiApplPluginStatus" => $plugin["status"],
				"cactiApplPluginVersion" => $plugin["version"]
			);
			snmpagent_cacheTableEntry_insert("CACTI-SMI", "cactiApplPluginTable", $i, $values );
			$i++;
		}
	}

	/* update total statistics */
	snmpagent_cachedMibGroup_refresh('CACTI-SMI', array('cactiStatsTotalsDataSources', 'cactiStatsTotalsGraphs', 'cactiStatsLastUpdate'));
}

function snmpagent_cachedMibGroup_refresh($mib, $objects){
	if(is_array($objects) && sizeof($objects)>0) {
		foreach($objects as $object) {
			snmpagent_cachedMibObject_refresh($mib, $object);
		}
	}
}

function snmpagent_cachedMibTable_refresh($mib, $table){
	if($mib == "CACTI-SMI") {
		switch($table) {
			case "cactiStatsTotalsDeviceStatusTable":
				$devicestatus_indices = array( 0=> 0, 1=> 1, 2=> 2, 3=> 3, 4=> 4);
				$current_states = db_fetch_assoc("SELECT status, COUNT(*) as cnt FROM `host` GROUP BY status");
				if($current_states && sizeof($current_states)>0) {
					foreach($current_states as $current_state) {
						$index = $devicestatus_indices[$current_state["status"]];
						$values = array(
							"cactiStatsTotalsDeviceStatusIndex" => $current_state["status"],
							"cactiStatsTotalsDeviceStatusCounter" => $current_state["cnt"]
						);
						snmpagent_cacheTableEntry_replace('CACTI-SMI', 'cactiStatsTotalsDeviceStatusTable', $index, $values);
						unset($devicestatus_indices[$current_state["status"]]);
					}
				}
				if(sizeof($devicestatus_indices)>0) {
					foreach($devicestatus_indices as $status => $index) {
						$values = array(
							"cactiStatsTotalsDeviceStatusIndex" => $status,
							"cactiStatsTotalsDeviceStatusCounter" => 0
						);
						snmpagent_cacheTableEntry_replace('CACTI-SMI', 'cactiStatsTotalsDeviceStatusTable', $index, $values);
					}
				}
				break;
		}
	}else {
		/* plugin related stuff here*/
	}
}

function snmpagent_cachedMibObject_refresh($mib, $object){
	if($mib == "CACTI-SMI") {
		switch($object) {
			case "cactiApplLastUpdate":
				$value = gmmktime();
				break;
			case "cactiApplVersion":
				$value = db_fetch_cell("SELECT `cacti` FROM `version`");
				break;
			case "cactiApplSnmpVersion":
				$versions = array("net-snmp" => 1, "ucd-snmp" => 2);
				$snmp_version = read_config_option("snmp_version", true);
				$value = $versions[$snmp_version];
				if(function_exists("snmpget")) {
					$value = 3;
				}
				break;
			case "cactiApplRrdtoolVersion":
				$value = read_config_option("rrdtool_version", true);
				break;
			case "cactiApplPollerEnabled":
				$value = (read_config_option("poller_enabled", true) == "on") ? 1 : 2;
				break;
			case "cactiApplPollerType":
				$value = read_config_option("poller_type", true);
				break;
			case "cactiApplPollerInterval":
				$value = read_config_option("poller_interval", true);
				break;
			case "cactiApplPollerMaxProcesses":
				$value = read_config_option("concurrent_processes", true);
				break;
			case "cactiApplPollerLoadBalance":
				$value = (read_config_option("process_leveling", true) == "on") ? 1 : 2;
				break;
			case "cactiApplSpineMaxThreads":
				$value = read_config_option("max_threads", true);
				break;
			case "cactiApplSpineScriptServers":
				$value = read_config_option("php_servers", true);
				break;
			case "cactiApplSpineScriptTimeout":
				$value = read_config_option("script_timeout", true);
				break;
			case "cactiApplSpineMaxOids":
				$value = read_config_option("max_get_size", true);
				break;
			case "cactiStatsLastUpdate":
				$value = gmmktime();
				break;
			case "cactiStatsTotalsDevices":
				$value = db_fetch_cell("SELECT COUNT(*) FROM host");
				break;
			case "cactiStatsTotalsDataSources":
				$value = db_fetch_cell("SELECT COUNT(*) FROM data_local");
				break;
			case "cactiStatsTotalsGraphs":
				$value = db_fetch_cell("SELECT COUNT(*) FROM graph_local");
				break;

		}
		snmpagent_cache_update($mib, $object, $value);
	}else {
		/* plugin related stuff here*/
	}
}

function snmpagent_cache_update($mib, $object, $value){
	db_execute("UPDATE `plugin_snmpagent_cache` SET `value` = '$value' WHERE `mib` = '$mib' AND `name` = '$object' LIMIT 1");
}

function snmpagent_cache_read($mib, $object){
	return db_fetch_row("SELECT * FROM plugin_snmpagent_cache WHERE name='$object' AND mib='$mib' LIMIT 1");
}

function snmpagent_cacheTableEntry_insert( $mib, $table, $index, $values){
	$oid_entry = snmpagent_cacheTableEntry_exists( $mib, $table, $index );
	if($oid_entry == false) {
		$columns = snmpagent_cacheTableColumns_list($mib, $table);
		if($columns & sizeof($columns)>0) {
			foreach($columns as $column_params) {
				$column_params["oid"] .= "." . $index;
				$column_params["otype"] = "DATA";
				if(isset($values[$column_params["name"]])) {
					$column_params["value"] = $values[$column_params["name"]];
				}
				db_execute("INSERT INTO `plugin_snmpagent_cache` (`oid`, `name`, `mib`, `type`, `otype`, `max-access`, `value`) VALUES ('"
					. $column_params["oid"] . "','"
					. $column_params["name"] . "','"
					. $column_params["mib"] . "','"
					. $column_params["type"] . "','"
					. $column_params["otype"] . "','"
					. $column_params["max-access"] . "','"
					. $column_params["value"] . "')");
			}
			return true;
		}
	}
	return false;
}

function snmpagent_cacheTableEntry_delete( $mib, $table, $index){
	$oid_entry = snmpagent_cacheTableEntry_exists( $mib, $table, $index );
	if($oid_entry != 'ERROR' && $oid_entry !== false) {
		/* get list of columns for this mib table */
		$columns = snmpagent_cacheTableColumns_list($mib, $table);
		if($columns & sizeof($columns)>0) {
			foreach($columns as $column_params) {
				$column_params["oid"] .= "." . $index;
				db_execute("DELETE FROM `plugin_snmpagent_cache` WHERE `oid` = '" . $column_params["oid"] . "' LIMIT 1");
			}
			return true;
		}
	}
	return false;
}

function snmpagent_cacheTableEntry_update( $mib, $table, $index, $values){
	$oid_entry = snmpagent_cacheTableEntry_exists( $mib, $table, $index );
	if($oid_entry != 'ERROR' && $oid_entry !== false) {
		$columns = snmpagent_cacheTableColumns_list($mib, $table);
		if($columns & sizeof($columns)>0) {
			foreach($columns as $column_params) {
				$column_params["oid"] .= "." . $index;
				if(isset($values[$column_params["name"]])) {
					db_execute("UPDATE `plugin_snmpagent_cache` SET `value` = '" . $values[$column_params["name"]] . "' WHERE `oid` = '" . $column_params["oid"] . "'");
				}
			}
			return true;
		}
	}
	return false;
}

function snmpagent_cacheTableEntry_replace( $mib, $table, $index, $values){
	$oid_entry = snmpagent_cacheTableEntry_exists( $mib, $table, $index );
	if($oid_entry != 'ERROR' ) {
		snmpagent_cacheTableEntry_delete( $mib, $table, $index);
		return snmpagent_cacheTableEntry_insert( $mib, $table, $index, $values);
	}
	return false;
}

function snmpagent_cacheTableEntry_exists( $mib, $table, $index ){
	$oid_table = db_fetch_cell("SELECT `oid` FROM `plugin_snmpagent_cache` WHERE `mib` = '" . $mib . "' AND `name` = '" . $table . "'");
	if(!$oid_table) {
		cacti_log("ERROR: Unknown MIB table '" . $table . "' does not exist.", false, 'SNMPAgent');
		return "ERROR";
	}
	$oid_entry = $oid_table . '.1';

	/* check if entry exists */
	$exists = db_fetch_cell("SELECT 1 FROM plugin_snmpagent_cache WHERE oid = '" . $oid_entry . ".1." . $index . "'");
	return ($exists) ? $oid_entry : false;
}

function snmpagent_cacheTableColumns_list( $mib, $table){
	$oid_table = db_fetch_cell("SELECT `oid` FROM `plugin_snmpagent_cache` WHERE `mib` = '" . $mib . "' AND `name` = '" . $table . "'");
	if(!$oid_table) {
		cacti_log("ERROR: Unknown MIB table '" . $table . "' does not exist.", false, 'SNMPAgent');
		return false;
	}
	/* As defined by SMI the OID value assigned to the row must be the same as the OID value assigned to the table containing
	   the row with addition of a single value of one. */
	return db_fetch_assoc("SELECT * FROM plugin_snmpagent_cache WHERE oid LIKE '" . $oid_table . ".1.%' GROUP BY name ORDER BY oid");
}

function snmpagent_cacheTable_truncate( $mib, $table){
	$oid_table = db_fetch_cell("SELECT `oid` FROM `plugin_snmpagent_cache` WHERE `mib` = '" . $mib . "' AND `name` = '" . $table . "'");
	if(!$oid_table) {
		cacti_log("ERROR: Unknown MIB table '" . $table . "' does not exist.", false, 'SNMPAgent');
		return "ERROR";
	}
	$oid_entry = $oid_table . '.1';
	db_execute("DELETE FROM `plugin_snmpagent_cache` WHERE `mib` = '" . $mib . "' AND `otype` = 'DATA' AND `oid` LIKE '" . $oid_entry . ".%'");
	return true;
}
